package cn.sh.stc.sict.theme.hphy.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.theme.hphy.constant.DataConstant;
import cn.sh.stc.sict.theme.hphy.dao.HpDeptInfoMapper;
import cn.sh.stc.sict.theme.hphy.dao.HpDocInfoMapper;
import cn.sh.stc.sict.theme.hphy.dao.HpHosInfoMapper;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HpHosInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDeptInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpHosInfoService;
import cn.sh.stc.sict.theme.hphy.vo.DeptDoctorsVO;
import cn.sh.stc.sict.theme.hphy.wd.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * (HpDeptInfo)表服务实现类
 *
 * @author makejava
 * @since 2021-01-28 10:35:58
 */
@Slf4j
@AllArgsConstructor
@Service("hpDeptInfoService")
public class HpDeptInfoServiceImpl extends ServiceImpl<HpDeptInfoMapper, HpDeptInfo> implements HpDeptInfoService {
    private final HpDocInfoService hpDocInfoService;
    private final HpHosInfoService hpHosInfoService;

    @Override
    public DeptDoctorsVO getDeptDoctors(String hospitalCode, String deptCode) {
        HpHosInfo hosInfo = hpHosInfoService.getById(hospitalCode);
        // 查询科室信息
        // 查询医生信息
        HpDeptInfo deptInfo = this.getOne(
                new LambdaQueryWrapper<HpDeptInfo>()
                        .eq(HpDeptInfo::getHosOrgCode, hospitalCode)
                        .and(i -> i.eq(HpDeptInfo::getOneDeptCode, deptCode)
                                .or().eq(HpDeptInfo::getDeptCode, deptCode))
                        .orderByDesc(HpDeptInfo::getDeptLevel)
                        .last("limit 1")
        );
        if (deptInfo == null) {
            log.error("根据医院编码：{} 和科室编码：{} 未查出科室信息", hospitalCode, deptCode);
            return null;
        }

        LambdaQueryWrapper<HpDocInfo> docWrapper = new LambdaQueryWrapper<>();
        docWrapper.eq(HpDocInfo::getHosOrgCode, hospitalCode);
        docWrapper.eq(HpDocInfo::getOneDeptCode, deptInfo.getOneDeptCode());
        if (DataConstant.TWO_DEPT.equals(deptInfo.getDeptLevel())) {
            docWrapper.eq(HpDocInfo::getDeptCode, deptInfo.getDeptCode());
        }
        docWrapper.orderByAsc(HpDocInfo::getIndexNo);

        List<HpDocInfo> doctors = hpDocInfoService.list(docWrapper);


        DeptInfo info = new DeptInfo();
        info.setHosOrgCode(deptInfo.getHosOrgCode());
        info.setOneDeptCode(deptInfo.getOneDeptCode());
        info.setDeptCode(deptInfo.getDeptCode());
        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(info);
        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(info);
        List<HpDocInfo> remainDoctors = this.getRemainDoctList(doctors, resourceDoctInfo);

        return new DeptDoctorsVO(hosInfo, deptInfo, resourceOutPatInfo, remainDoctors);
    }

    private List<HpDocInfo> getRemainDoctList(List<HpDocInfo> doctors, List<DoctInfo> resourceDoctInfo) {
        Map<String, HpDocInfo> doctorMap = doctors.stream().collect(Collectors.groupingBy(HpDocInfo::getResourceCode, Collectors.collectingAndThen(Collectors.toList(), x -> x.get(0))));
        List<HpDocInfo> remainDoctors = new ArrayList<>();
        resourceDoctInfo.forEach(res -> {
            if(doctorMap.containsKey(res.getResourceCode())){
                remainDoctors.add(doctorMap.get(res.getResourceCode()));
            }else{
                HpDocInfo doctor = BeanUtil.toBean(res, HpDocInfo.class);
                hpDocInfoService.save(doctor);
                remainDoctors.add(doctor);
            }
        });

        return remainDoctors;
    }

//    public static void main(String[] args) {
//
//        DeptInfo info = new DeptInfo();
//        info.setHosOrgCode("42503527600");
//        info.setOneDeptCode("1013");
//        info.setDeptCode("3122");
//        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(info);
//        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(info);
//        log.error("OutPatInfo = {}", JSONUtil.toJsonStr(resourceOutPatInfo));
//        log.error("=========================");
//        log.error("DoctInfo = {}", JSONUtil.toJsonStr(resourceDoctInfo));
//
//        DoctInfo doctInfo = resourceDoctInfo.get(0);
//        NumSourceInfo numSourceInfo = new NumSourceInfo();
//        numSourceInfo.setHosOrgCode(doctInfo.getHosOrgCode());
//        numSourceInfo.setOneDeptCode(doctInfo.getOneDeptCode());
//        numSourceInfo.setDeptCode(doctInfo.getDeptCode());
//        numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
//        numSourceInfo.setResourceCode(doctInfo.getResourceCode());
//        numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
//        numSourceInfo.setStartTime("2022-07-15");
//        numSourceInfo.setEndTime("2022-07-22");
//        numSourceInfo.setOrderNumType("102");
//
//        List<NumSourceInfo> orderNumInfo = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
//        log.error("=========================");
//        log.error("numInfo = {}", JSONUtil.toJsonStr(orderNumInfo));
//    }


//    public static void main(String[] args) {
//        NumSourceInfo numSourceInfo = new NumSourceInfo();
//        numSourceInfo.setHosOrgCode("42503527600");
//        numSourceInfo.setOneDeptCode("1013");
//        numSourceInfo.setDeptCode("3122");
//        numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
//        numSourceInfo.setResourceCode("10220");
//        numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
//        numSourceInfo.setStartTime("2022-07-15");
//        numSourceInfo.setEndTime("2022-07-22");
//        numSourceInfo.setOrderNumType("102");
//
//        String orderNumType = numSourceInfo.getOrderNumType();
//        if(WanDaConstant.ORDER_TYPE_OUTP.equals(numSourceInfo.getOrderType())){
//            numSourceInfo.setOrderNumType(null);
//        }
//        List<NumSourceInfo> list = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
//        if(WanDaConstant.ORDER_TYPE_OUTP.equals(numSourceInfo.getOrderType())){
//            List<NumSourceInfo> result = list.stream().filter(i -> orderNumType.equals(i.getOrderNumType())).collect(Collectors.toList());
//            log.error("result = {}", JSONUtil.toJsonStr(result));
//        }else{
//            log.error("list = {}", JSONUtil.toJsonStr(list));
//        }
//    }
}
